#--
# Copyright (c) 2009-2010 Novell, Inc.
#
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of version 2 of the GNU General Public License
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail,
# you may find current contact information at www.novell.com
#++
# = Administrator controller
# Provides access to configuration of system administrator.

class AdministratorController < ApplicationController

  public

  def index
    authorize! :read, Administrator
    @write_permission = can? :write, Administrator

    @administrator	= Administrator.find
    @administrator.confirm_password	= ""
    params[:firstboot] = 1 if Basesystem.new.load_from_session(session).in_process?

    respond_to do |format|
      format.html
      format.xml  { render :xml => @administrator.to_xml(:dasherize => false) }
      format.json { render :json => @administrator.to_json }
    end
  end

  def update
    authorize! :write, Administrator
    admin = params["administrator"]

    head :unprocessable_entity and return if admin.blank?

    save_aliases = params['submit_type'] == 'save_aliases'
    #validate data also here, if javascript in view is off
    if (admin["aliases"].present? && admin["aliases"] != "NONE") || save_aliases
      admin["aliases"].split(",").each do |mail|
        #only check emails, not local users
        unless mail.match /\A.+@.+$/ #only trivial check
          problem _("Enter a valid e-mail address.")
          return
        end
      end
    end
    # we cannot pass empty string to rest-service
    # TODO FIXME: is this 'NONE' workaround still needed?
    aliases  = admin['aliases'].empty? ? 'NONE' : admin['aliases']
    password = admin['password']
    password_confirmation = admin['confirm_password']
    if save_aliases
      # only save selected subset of administrator data:
      password = nil
    elsif password != password_confirmation
      problem _("Passwords do not match.")
      return
    end
    @administrator = Administrator.find
    @administrator.password = password
    @administrator.aliases  = aliases
    begin
      @administrator.save
      flash[:notice] = _('Administrator settings have been written.')
    rescue Exception => error
      problem _("Error while saving administrator settings.")
      Rails.logger.error "ERROR: #{error.inspect}"
      Rails.logger.error "backtrace: #{error.backtrace.join("\n")}"

      return
    end

    # check if mail is configured; during initial workflow, only warn if mail configuration does not follow
    if admin["aliases"] != "" && (defined?(Mail) == 'constant' && Mail.class == Class) &&
        (!Basesystem.new.load_from_session(session).following_steps.any? { |h| h[:controller] == "mail" })
      @mail = Mail.find :one
      if @mail && (@mail.smtp_server.nil? || @mail.smtp_server.empty?)
        flash[:warning] = _("Mail alias was set but outgoing mail server is not configured.")
      end
    end

    respond_to do |format|
      format.html { redirect_success }
      format.xml  { render :xml => @administrator.to_xml(:dasherize => false) }
      format.json { render :json => @administrator }
    end
  end

  # See update
  def create
    update
  end

private
  def problem message
    if request.format.html?
      flash[:error] = message
      redirect_to :action => "index"
    else #REST request
      error = { "error" => { "type" => "ADMINISTRATOR_ERROR", "messsage" => message, "id" => "ADMINISTRATOR" }}
      respond_to do |format|
        format.xml  { render :xml => error, :status => 400 }
        format.json { render :json => error, :status => 400 }
      end
    end
  end
end
